/* Copyright (C) 2012 Chris Vine

The library comprised in this file or of which this file is part is
distributed by Chris Vine under the GNU Lesser General Public
License as follows:

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License, version 2.1, for more details.

   You should have received a copy of the GNU Lesser General Public
   License, version 2.1, along with this library (see the file LGPL.TXT
   which came with this source code package in the c++-gtk-utils
   sub-directory); if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA, 02111-1307, USA.
*/

#include <glib.h>
#include <vector>
#include <algorithm>

#include <c++-gtk-utils/mem_fun.h>

using namespace Cgu;

class Test {
  int multiplier;
public:
  int multiply(int i) const {return i * multiplier;}
  Test(int i): multiplier(i) {}
};


extern "C" {
static void test_mem_fun() {
  
  int mult = 2;
  Test t(mult);
  const Test t_const(mult);

  std::vector<int> in;
  for (int i = 0; i < 5; ++i)
    in.push_back(i);

  // non-const Test object
  std::vector<int> out;
  std::transform(in.begin(), in.end(),
		 std::back_inserter(out),
		 MemFun::make(t, &Test::multiply));
  for (int i = 0; i < 5; ++i)
    g_assert_cmpint(out[i], ==, i * mult);

  // const Test object
  out.clear();
  std::transform(in.begin(), in.end(),
		 std::back_inserter(out),
		 MemFun::make(t_const, &Test::multiply));
  for (int i = 0; i < 5; ++i)
    g_assert_cmpint(out[i], ==, i * mult);
}
} // extern "C"


int main (int argc, char* argv[]) {
  g_test_init(&argc, &argv, static_cast<void*>(0));

  g_test_add_func("/mem_fun/mem_fun", test_mem_fun); 

  return g_test_run();
}
